%% ************************************************************************************
% Correct Triangular algorithm, Carriero Chan Clark and Marcellino (2021),
% Corrigendum to "Large Vector Autoregressions 
% with stochastic volatility and flexible priors."
% ************************************************************************************
% Model is:
%
%     Y(t) = Pi(L)Y(t-1) + v(t); Y(t) is Nx1; 
%     v(t) = inv(A)*(LAMBDA(t)^0.5)*e(t); e(t) ~ N(0,I);
%                   _                                         _
%                  |    1          0       0       ...      0  |
%                  |  A(2,1)       1       0       ...      0  |
%         A =      |  A(3,1)     A(3,2)    1       ...      0  |
%                  |   ...        ...     ...      ...     ... |
%                  |_ A(N,1)      ...     ...   A(N,N-1)    1 _|
%
%    Lambda(t)^0.5 = diag[sqrt_h(1,t)  , .... , sqrt_h(N,t)];
%
%    ht=exp(Vol_states)
%    sqrt_ht=sqrt(ht)=sqrt(exp(Vol_states))=exp(Vol_states/2)
%    Vol_states=2*ln(sqrt_ht)
%
%    Vol_states(i,t)   = Vol_states(i,t-1) + eta(i,t),  
%    eta(t) ~ N(0,PHI); with PHI full 
% ------------------------------------------------------------------------------------

%% Initial operations 
clear all; close all; clc;   %#ok<CLALL>
seed = RandStream('mt19937ar','Seed',3211321);
RandStream.setGlobalStream(seed);
scrsz = get(groot,'ScreenSize');

%% Load Data
% load CSV file
dum=importdata('Example_data.csv',',');
% Dates, start March 1959=1959.25
dates=dum.data(2:end,1);
% Variable names
ncode=dum.textdata(1,2:end);
% Transformation codes (data are already transformed)
tcode=dum.data(1,2:end);
% Data
data=dum.data(2:end,2:end);
   
%% --------------------------- OPTIONS ----------------------------------
p = 13;                      % Number of lags on dependent variables
check_stationarity=0;        % Truncate nonstationary draws? (1=yes)
theta=[0.2^2 0.5^2 100 2];   % hyperparameters of Minnesota prior: 
                             % [lambda1 lambda2 int lambda3], int is the 
                             % prior on the intercept. lambda1, lambda2
                             % and lambda3 are as in equation (42) with
                             % lambda1 the overall shrinkage, lambda2 the
                             % cross srhinkage and lambda 3 the lag decay
                             % (quadratic if =2). Note lambda2~=1 implies
                             % the prior becomes asymmetric across eqation,
                             % so this would not be implementable in the
                             % standard conjugate setup. 
Minn_pmean = 0;              % Prior mean of the 1-st own lag for each 
                             % equation. For nonstationary variables, this
                             % is usually set to 1. For transformed
                             % stationary variables this is set to 0. 
                             
M          = 20000; 1000;           % Final number of draws after burn in
burn_in    = 0.1*M;          % burn in
MCMCreps   = M + burn_in;    % total MCMC draws
Compute_diagnostics=1;       % compute Inefficiency Factors and Potential 
                             % Scale Reduction Factors (1=yes) (takes time).

%% -------------------------Create data matrices-------------------------
% pointers
[T,N]=size(data);
% matrix X
lags=zeros(T,N*p);
for l=1:p; lags(p+1:T,(N*(l-1)+1):N*l)=data(p+1-l:T-l,1:N); end
X = [ones(T-p,1) lags(p+1:T,:)];
% trim Y 
Y = data(p+1:end,:); 
% update pointers
[T,K]=size(X);

%% -----------------Prior hyperparameters for bvar model

% Prior on conditional mean coefficients, use Minnesota setup
ARresid=[];
for i=1:N
    yt_0=[ones(T-1,1) Y(1:end-1,i)];
    yt_1=Y(2:end,i);
    ARresid(:,i)=yt_1-yt_0*(yt_0\yt_1); %#ok<SAGROW>
end
AR_s2= diag(diag(ARresid'*ARresid))./(T-2);

Pi_pm=zeros(N*(K-1),1); Pi_pv=eye(N*(K-1)); co=0;
for i=1:N
    sigma_const(i)=AR_s2(i,i)*theta(3); %#ok<SAGROW> % this sets the prior variance on the intercept 
    for l=1:p
        for j=1:N
            co=co+1;
            if (i==j)
                Pi_pv(co,co)=theta(1)/(l^theta(4)); % prior variance, own lags
                if l==1; Pi_pm(co)=Minn_pmean; end % this sets the prior means for the first own lag coefficients.
            else
                Pi_pv(co,co)=(AR_s2(i,i)/AR_s2(j,j)*theta(1)*theta(2)/(l^theta(4))); % prior variance, cross-lags
            end
        end
    end
end

% Pai~N(vec(MU_pai),OMEGA_pai), equation 7. 
OMEGA_pai   = diag(vec([sigma_const;reshape(diag(Pi_pv),K-1,N)])); % prior variance of Pai 
MU_pai      = [zeros(1,N);reshape(Pi_pm,K-1,N)];                   % prior mean of Pai 

% A~N(MU_A,inv(OMEGA_A_inv)), equation 8.
for i = 2:N
    MU_A(1:i-1,i) = zeros(i-1,1);            %#ok<SAGROW> % prior mean of A 
    OMEGA_A_inv(1:i-1,1:i-1,i) = 0*eye(i-1); %#ok<SAGROW> % prior precision of A 
end

% PHI~IW(s_PHI,d_PHI), equation 9. 
d_PHI = N+3;                 % prior dofs
s_PHI = d_PHI*(0.15*eye(N)); % prior scale, where eye(N)=PHI_ in equation (9)

% prior on initial states
Vol_0mean = zeros(N,1);   % time 0 states prior mean
Vol_0var  = 100*eye(N);   % time 0 states prior variance

                           
    %% >>>>>>>>>>>>>>>>>>>>>>>>>> Gibbs sampler <<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    % Storage arrays for posterior draws
    PAI_all     = zeros(M,K,N); PHI_all     = zeros(M,N*(N-1)/2+N);
    invA_all    = zeros(M,N,N); sqrtht_all  = zeros(M,T,N);
    
    % define some useful matrices out of the loop
    PAI = zeros(K,N);                                         % pre-allocate space for PAI
    comp=[eye(N*(p-1)),zeros(N*(p-1),N)];                     % companion form
    iV=diag(1./diag(OMEGA_pai)); iVb_prior=iV*vec(MU_pai);    % inverses of prior matrices
    
    % initializations
    A_         = eye(N);                                      % initialize A matrix 
    invA_      = inv(A_);                                     % initialize inv(A) matrix
    sqrt_ht    = sqrt([ARresid(1,:).^2; ARresid.^2]);         % Initialize sqrt_sqrtht
    Vol_states = 2*log(sqrt_ht);                              % Initialize states
    PHI_       = 0.0001*eye(N);                               % Initialize PHI_, a draw from the covariance matrix W
        
    %% start of MCMC loop  
    % the algorithm is as described in page 15, but it starts from step 2b,
    % this is the same as starting from step 1 but is more convenient as
    % it requires less initializations (one can think of steps 2b to 2d in
    % repetition 1 as an initialization).
    
    tic;
    for m = 1:MCMCreps 
        
        if mod(m,10) == 0; clc; disp(['percentage completed:' num2str(100*m/MCMCreps) '%']); toc; end 
        
        %% STEP 2b: Draw from the conditional posterior of PAI, equation 10.
        if m==1; PAI= X\Y; end 
        stationary=0;
        while stationary==0
            
            % This is the only new step (triangular algorithm).
            PAI=CTA(Y,X,N,K,T,A_,sqrt_ht,iV,iVb_prior,PAI);  
            
        if (check_stationarity==0 || max(abs(eig([PAI(2:K,:)' ; comp]))) < 1); stationary = 1; end
        end
        RESID = Y - X*PAI; % compute the new residuals
            
        %% STEP 2c: Draw the covariances, equation 11. 
        for ii = 2:N
            % weighted regression to get Z'Z and Z'z (in Cogley-Sargent 2005 notation)
            y_spread_adj=RESID(:,ii)./sqrt_ht(:,ii);
            X_spread_adj=[]; for vv=1:ii-1;  X_spread_adj=[X_spread_adj RESID(:,vv)./sqrt_ht(:,ii)]; end  %#ok<AGROW>
            ZZ=X_spread_adj'*X_spread_adj; Zz=X_spread_adj'*y_spread_adj;
            % computing posteriors moments
            Valpha_post = (ZZ + OMEGA_A_inv(1:ii-1,1:ii-1,ii))\eye(ii-1);
            alpha_post  = Valpha_post*(Zz + OMEGA_A_inv(1:ii-1,1:ii-1,ii)*MU_A(1:ii-1,ii));
            % draw and store 
            alphadraw   = alpha_post+chol(Valpha_post,'lower')*randn(ii-1,1);
            A_(ii,1:ii-1)= -1*alphadraw';
        end 
        invA_=A_\eye(N); % compute implied draw from A^-1, needed in step 2b. 
              
        %% STEP 2d and STEP 1: Draw mixture states and then volatility states
        Vol_states  = KSC(log((RESID*A_').^2 + 1e-6),Vol_states,PHI_,Vol_0mean,Vol_0var); % use KSC algorithm 
        sqrt_ht  = exp(Vol_states/2); %compute sqrtht^0.5 from volatility states, needed in step 2b 
              
        %% STEP 2a: Draw volatility variances
        eta  = Vol_states(2:end,:) - Vol_states(1:end-1,:); % innovations to volatility states, using equation 5.
        temp = chol(inv(s_PHI + eta'*eta),'lower')*randn(N,T + d_PHI); % compute draw from inv(PHI).
        PHI_ = inv(temp*temp'); % derive posterior draw from PHI, equation 12.             
        
        %% Store the posterior draws 
        if m > burn_in
        PAI_all(m-burn_in,:,:)     = PAI;         PHI_all(m-burn_in,:,:)     = PHI_((tril(PHI_))~=0);
        invA_all(m-burn_in,:,:)    = invA_;       sqrtht_all(m-burn_in,:,:)  = sqrt_ht;
        end

    end %end of the Gibbs sampler
    
    toc2=toc;
    disp(['algorithm 1 took ' num2str(toc2) ' seconds'])

%% Convergence diagnostics
if Compute_diagnostics==1
disp('computing convergence diagnostics..')
Diagnostics(sqrtht_all,invA_all,PAI_all,PHI_all,N,K,M);
end

%% Graph of volatilities
disp('making volatility graphs..');
close all;

% compute volatilities
vola=zeros(size(sqrtht_all));
for m=1:M
    invA= squeeze(invA_all(m,:,:));
    for t=1:T
        vola(m,t,:) = sqrt(diag(invA*diag(squeeze(sqrtht_all(m,t,:)).^2)*invA'));
    end
end

% plot the volatilities
ff=figure(2); set(ff,'name','diagonal of Sigma(t)','Position',[scrsz(1) scrsz(4)/2 scrsz(3)/2.5 scrsz(4)*0.9])
figure(ff)
for i=1:N
    subplot(5,4,i,'FontSize',6)
    plot(dates(p+1:end),quantile(vola(:,:,i),0.05),'b:',dates(p+1:end),quantile(vola(:,:,i),0.5 ),'r-',...
         dates(p+1:end),quantile(vola(:,:,i),0.95),'b:');
    hold on; axis tight
    title(ncode(i))
end

%% Compute impulse responses (IRFs)
disp('making impulse responses..');
FEDFUNDS=13;                       % index of FEDFUNDS 
nhor=60;                           % horizon for responses
RESPONSE  = zeros(M,N,nhor);       % storage of responses
RESPONSEc = zeros(M,N,nhor);       % storage of cumulated responses

    % compute the responses
    for m=1:M
        
        % rearrange stuff
        temp=squeeze(PAI_all(m,:,:)); 
        for i=1:N
            for l=1:p
                B(i,:,l)=temp(1+N*(l-1)+1:1+(N*(l-1)+N),i); %#ok<SAGROW>
            end
        end
        [neq,nvar,nlag]=size(B);
        
        % compute impulse responses (adapted from Sims' code)
        Hsd=squeeze(invA_all(m,:,:)); %USING UNIT SHOCKS so no multiplication by h(t)
        response=zeros(nvar,neq,nhor);
        response(:,:,1)=Hsd; % need lower triangular, last innovation untransformed
        for h=2:nhor
           for l=1:min(nlag,h-1)
              response(:,:,h)=response(:,:,h)+B(:,:,l)*response(:,:,h-l);
           end
        end
 
        % store the response to FEDFUNDS
        RESPONSE(m,:,:) = response(:,FEDFUNDS,:);
    end
    
    % cumulate responses for appropriate variables (undo transformations)
    for iv=1:N
        if  tcode(iv)==0 || (iv==FEDFUNDS || tcode(iv)==1 || tcode(iv)==4)
            RESPONSEc(:,iv,:)=RESPONSE(:,iv,:);
        else
            RESPONSEc(:,iv,1)=RESPONSE(:,iv,1);
            for h=2:nhor; RESPONSEc(:,iv,h)=RESPONSEc(:,iv,h-1)+RESPONSE(:,iv,h); end
            display(strcat('response for    ' ,'  ', ncode(iv), ' has been cumulated'))
        end
    end

    
%% IRFs graph 

% setup figure
ff=figure(1); set(ff,'name','responses to monetary policy shock','Position',[scrsz(1) scrsz(4)/2 scrsz(3)/2.5 scrsz(4)*0.9])
figure(ff);

% get quantiles and do the graph
selectquant=[.16, .5, .84];
impXY_1=squeeze(quantile(RESPONSEc,selectquant));
figure(1)
for ig=1:N
    subplot(5,4,ig)
    plot(1:nhor,squeeze(impXY_1(1,ig,:)),'b:',1:nhor,squeeze(impXY_1(2,ig,:)),'r-',...
         1:nhor,squeeze(impXY_1(3,ig,:)),'b:');
    title(ncode(ig)); xlim([1 nhor]); axis tight
end
    
%% end
disp(['algorithm 1 took ' num2str(toc2) ' seconds'])

